@@ -39,8 +39,19 @@ import yaml
 import minidb
 import logging
 
+try:
+    import msgpack
+except ImportError:
+    msgpack = None
+
+try:
+    import redis
+except ImportError:
+    redis = None
+
 from .util import atomic_rename, edit_file
 from .jobs import JobBase, UrlJob, ShellJob
+from .filters import FilterBase
 
 logger = logging.getLogger(__name__)
 
@@ -99,6 +110,7 @@ DEFAULT_CONFIG = {
             'device': None,
             'sound': 'spacealarm',
             'user': '',
+            'priority': 'normal',
         },
         'pushbullet': {
             'enabled': False,
@@ -262,12 +274,25 @@ class UrlsBaseFileStorage(BaseTextualFileStorage, metaclass=ABCMeta):
     def load_secure(self):
         jobs = self.load()
 
+        def is_shell_job(job):
+            if isinstance(job, ShellJob):
+                return True
+
+            for filter_kind, subfilter in FilterBase.normalize_filter_list(job.filter):
+                if filter_kind == 'shellpipe':
+                    return True
+
+                if job.diff_tool is not None:
+                    return True
+
+            return False
+
         # Security checks for shell jobs - only execute if the current UID
         # is the same as the file/directory owner and only owner can write
         shelljob_errors = self.shelljob_security_checks()
-        if shelljob_errors and any(isinstance(job, ShellJob) for job in jobs):
+        if shelljob_errors and any(is_shell_job(job) for job in jobs):
             print(('Removing shell jobs, because %s' % (' and '.join(shelljob_errors),)))
-            jobs = [job for job in jobs if not isinstance(job, ShellJob)]
+            jobs = [job for job in jobs if not is_shell_job(job)]
 
         return jobs
 
@@ -320,7 +345,8 @@ class UrlsYaml(BaseYamlFileStorage, UrlsBaseFileStorage):
         filename = args[0]
         if filename is not None and os.path.exists(filename):
             with open(filename) as fp:
-                return [JobBase.unserialize(job) for job in yaml.load_all(fp, Loader=yaml.SafeLoader) if job is not None]
+                return [JobBase.unserialize(job) for job in yaml.load_all(fp, Loader=yaml.SafeLoader)
+                        if job is not None]
 
     def save(self, *args):
         jobs = args[0]
@@ -463,8 +489,11 @@ class CacheMiniDBStorage(CacheStorage):
         return (guid for guid, in CacheEntry.query(self.db, minidb.Function('distinct', CacheEntry.c.guid)))
 
     def load(self, job, guid):
-        for data, timestamp, tries, etag in CacheEntry.query(self.db, CacheEntry.c.data // CacheEntry.c.timestamp // CacheEntry.c.tries // CacheEntry.c.etag,
-                                                             order_by=minidb.columns(CacheEntry.c.timestamp.desc, CacheEntry.c.tries.desc),
+        for data, timestamp, tries, etag in CacheEntry.query(self.db,
+                                                             CacheEntry.c.data // CacheEntry.c.timestamp
+                                                             // CacheEntry.c.tries // CacheEntry.c.etag,
+                                                             order_by=minidb.columns(CacheEntry.c.timestamp.desc,
+                                                                                     CacheEntry.c.tries.desc),
                                                              where=CacheEntry.c.guid == guid, limit=1):
             return data, timestamp, tries, etag
 
@@ -475,9 +504,10 @@ class CacheMiniDBStorage(CacheStorage):
         if count < 1:
             return history
         for data, timestamp in CacheEntry.query(self.db, CacheEntry.c.data // CacheEntry.c.timestamp,
-                                                order_by=minidb.columns(CacheEntry.c.timestamp.desc, CacheEntry.c.tries.desc),
+                                                order_by=minidb.columns(CacheEntry.c.timestamp.desc,
+                                                                        CacheEntry.c.tries.desc),
                                                 where=(CacheEntry.c.guid == guid)
-                                                & ((CacheEntry.c.tries == 0) | (CacheEntry.c.tries == None))):  # noqa
+                                                & ((CacheEntry.c.tries == 0) | (CacheEntry.c.tries == None))):  # noqa:E711
             if data not in history:
                 history[data] = timestamp
                 if len(history) >= count:
@@ -502,3 +532,72 @@ class CacheMiniDBStorage(CacheStorage):
             return result
 
         return 0
+
+
+class CacheRedisStorage(CacheStorage):
+    def __init__(self, filename):
+        super().__init__(filename)
+
+        if redis is None or msgpack is None:
+            raise ImportError('redis + msgpack are missing')
+
+        self.db = redis.from_url(filename)
+
+    def _make_key(self, guid):
+        return 'guid:' + guid
+
+    def close(self):
+        self.db.connection_pool.disconnect()
+        self.db = None
+
+    def get_guids(self):
+        guids = []
+        for guid in self.db.keys(b'guid:*'):
+            guids.append(str(guid[len('guid:'):]))
+        return guids
+
+    def load(self, job, guid):
+        key = self._make_key(guid)
+        data = self.db.lindex(key, 0)
+
+        if data:
+            r = msgpack.unpackb(data)
+            return r['data'], r['timestamp'], r['tries'], r['etag']
+
+        return None, None, 0, None
+
+    def get_history_data(self, guid, count=1):
+        history = {}
+        if count < 1:
+            return history
+
+        key = self._make_key(guid)
+        for i in range(0, self.db.llen(key)):
+            r = self.db.lindex(key, i)
+            c = msgpack.unpackb(r)
+            if (c['tries'] == 0 or c['tries'] is None):
+                if c['data'] not in history:
+                    history[c['data']] = c['timestamp']
+                    if len(history) >= count:
+                        break
+        return history
+
+    def save(self, job, guid, data, timestamp, tries, etag=None):
+        r = {
+            'data': data,
+            'timestamp': timestamp,
+            'tries': tries,
+            'etag': etag,
+        }
+        self.db.lpush(self._make_key(guid), msgpack.packb(r, use_bin_type=True))
+
+    def delete(self, guid):
+        self.db.delete(self._make_key(guid))
+
+    def clean(self, guid):
+        key = self._make_key(guid)
+        i = self.db.llen(key)
+        if self.db.ltrim(key, 0, 0):
+            return i - self.db.llen(key)
+
+        return 0
